require File.dirname(__FILE__) + '/../test_helper'
require 'items_controller'

# Re-raise errors caught by the controller.
class ItemsController; def rescue_action(e) raise e end; end

class ItemsControllerTest < Test::Unit::TestCase
  fixtures :items, :users

  def setup
    @controller = ItemsController.new
    @request    = ActionController::TestRequest.new
    @response   = ActionController::TestResponse.new
    @request.session[:user] = users(:quentin).id
  end

  def test_login_required
    @request.session[:user] = nil
    get :index
    assert_redirected_to :controller => "account", :action => "login"
  end

  def test_index
    get :index
    assert_response :success
    assert_template "index"
    assert_equal [items(:todo2), items(:todo)], assigns(:todo_items)
    assert_equal [items(:done)], assigns(:done_items)
  end

  def test_create
    count = Item.count
    post :create, :item => {:content => "fugafuga"}
    assert_response :success
    assert_equal count + 1, Item.count
    item = Item.find(:first, :order => "id desc")
    assert_equal 3, item.sort
    assert_equal "fugafuga", item.content
    assert_equal users(:quentin), item.user
  end

  def test_update_content
    post :update_content, :id => items(:todo).id, :value => "ほげほげ"
    assert_response :success
    assert_equal "ほげほげ", @response.body
    assert_equal "ほげほげ", items(:todo, true).content
  end

  def test_update_done_to_true
    post :update_done, :id => items(:todo).id, :done => "1"
    assert_response :success
    item = items(:todo, true)
    assert item.done?
    assert_equal 2, item.sort
  end

  def test_update_done_to_false
    post :update_done, :id => items(:done).id, :done => "0"
    assert_response :success
    item = items(:done, true)
    assert !item.done?
    assert_equal 3, item.sort
  end

  def test_destroy
    item = items(:todo)
    post :destroy, :id => item.id
    assert_response :success
    assert_raise(ActiveRecord::RecordNotFound) do
      Item.find(item.id)
    end
  end

  def test_sort
    # 順序を入れ替える
    post :sort, :items => [items(:todo).id, items(:todo2).id]
    assert_response :success
    # 下にある項目の方が sort が小さい
    assert_equal 1, items(:todo2, true).sort
    assert_equal 2, items(:todo, true).sort
  end
end
